@@ -1,3 +1,5 @@
+from __future__ import division
+
 from numpy.testing import assert_almost_equal
 from statsmodels.datasets import star98
 from statsmodels.emplike.elanova import ANOVA
@@ -8,10 +10,11 @@ class TestANOVA(object):
     Tests ANOVA difference in means
     """
 
-    def __init__(self):
-        self.data = star98.load().exog[:30, 1:3]
-        self.res1 = ANOVA([self.data[:, 0], self.data[:, 1]])
-        self.res2 = ANOVAResults()
+    @classmethod
+    def setup_class(cls):
+        cls.data = star98.load().exog[:30, 1:3]
+        cls.res1 = ANOVA([cls.data[:, 0], cls.data[:, 1]])
+        cls.res2 = ANOVAResults()
 
     def test_anova(self):
         assert_almost_equal(self.res1.compute_ANOVA()[:2],
